<!DOCTYPE html>
<!--[if lte IE 8 ]> <html class="ie8" lang="en"> <![endif]-->
<!--[if (gt IE 8)|!(IE)]><!--> <html lang="en"> <!--<![endif]-->
<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="description" content="">
    <meta name="author" content="">
    <title>phpSysInfo</title>
    <link href="gfx/favicon.ico" type="image/x-icon" rel="shortcut icon">
    <link rel="stylesheet" type='text/css' href="templates/vendor/bootstrap.min.css">
    <link rel="stylesheet" type='text/css' href="templates/misc/emptyfile.css" title="PSI_Template">
    <!-- HTML5 Shim and Respond.js IE6-8 support of HTML5 elements and media queries -->
    <!--[if lt IE 9]>
    <script type="text/JavaScript" src="./js.php?name=html5shiv-printshiv"></script>
    <script type="text/JavaScript" src="./js.php?name=respond"></script>
    <![endif]-->
</head>

<body>

<input type="hidden" id="plugins" value="<?php $plugins = ''; foreach (CommonFunctions::getPlugins() as $plugin) : $plugins .= $plugin.','; endforeach; echo substr($plugins, 0, -1);?>"/>
<input type="hidden" id="showCPUListExpanded" value="<?php echo $showCPUListExpanded;?>"/>
<input type="hidden" id="showCPUInfoExpanded" value="<?php echo $showCPUInfoExpanded;?>"/>
<input type="hidden" id="showNetworkInfosExpanded" value="<?php echo $showNetworkInfosExpanded;?>"/>
<input type="hidden" id="showNetworkActiveSpeed" value="<?php echo $showNetworkActiveSpeed;?>"/>
<input type="hidden" id="showCPULoadCompact" value="<?php echo $showCPULoadCompact;?>"/>
<input type="hidden" id="blocks" value="<?php echo $blocks;?>"/>

<div class="navbar navbar-fixed-top">
    <div class="container">
        <div class="navbar-brand">
            <img class="logo" alt="" src="gfx/logo_32.gif" title="reload" style="display:inline; width:32px;"/> phpSysInfo <sup><a title="phpSysInfo - <?php echo PSI_VERSION_STRING; ?>" class="psihref" href="http://phpsysinfo.sourceforge.net/" target="psihref"><?php echo PSI_VERSION; ?></a></sup>
            <img id="loader" alt="" src="gfx/ajax-loader.gif" style="display:inline; visibility: hidden;"/>
            <img class="errorbutton" id="errorbutton" alt="" src="gfx/attention.gif" title="errors" data-toggle="modal" data-target="#errors-dialog" style="display:inline; visibility: hidden;"/>
        </div>
        <div id="select" class="select" style="text-align:right;padding-top:20px;display:none;">
            <span id="lang_044" style="display:none;">Template</span>
            <select class="template" id="template" style="display:none;">
<?php if ($picktemplate) {
foreach ($bootstraptemplates as $t) :
    $selected = "";
    if ($bootstraptemplate === $t) {
        $selected = " selected=\"selected\"";
    }
    echo "                <option value=\"".$t."\"".$selected.">".$t."</option>\n";
endforeach;
} else {
    echo "                <option value=\"".$bootstraptemplate."\" selected=\"selected\">".$bootstraptemplate."</option>\n";
}?>
            </select>
            <span id="lang_045" style="display:none;">Language</span>
            <select class="language" id="language" style="display:none;">
<?php if ($picklanguage) {
foreach ($languages as $l) :
    $selected = "";
    if ($language === $l) {
        $selected = " selected=\"selected\"";
    }
    echo "                <option value=\"".$l."\"".$selected.">".$l."</option>\n";
endforeach;
} else {
    echo "                <option value=\"".$language."\" selected=\"selected\">".$language."</option>\n";
}?>
            </select>
        </div>
    </div>
</div>

<div class="container">
<div id="output" class="row" style="display:none;">
    <div id="block_vitals" class="col-lg-6" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_002">System vitals</span></div>
            <div class="panel-body">
                <table id="vitals" class="table table-hover table-condensed noborderattop">
                    <tbody>
                        <tr>
                            <th><span id="lang_003">Hostname</span></th>
                            <td><span data-bind="Hostname"></span></td>
                        </tr>
                        <tr>
                            <th><span id="lang_004">Listening IP</span></th>
                            <td><span data-bind="IPAddr"></span></td>
                        </tr>
                        <tr>
                            <th><span id="lang_005">Kernel Version</span></th>
                            <td><span data-bind="Kernel"></span></td>
                        </tr>
                        <tr>
                            <th><span id="lang_006">Distro Name</span></th>
                            <td><span data-bind="Distro"></span></td>
                        </tr>
                        <tr>
                            <th><span id="lang_007">Uptime</span></th>
                            <td><span data-bind="Uptime"></span></td>
                        </tr>
                        <tr>
                            <th><span id="lang_095">Last boot</span></th>
                            <td><span data-bind="LastBoot"></span></td>
                        </tr>
                        <tr>
                            <th><span id="lang_008">Current Users</span></th>
                            <td><span data-bind="Users"></span></td>
                        </tr>
                        <tr>
                            <th><span id="lang_009">Load Averages</span></th>
                            <td><span data-bind="LoadAvg"></span></td>
                        </tr>
                        <tr id="tr_SysLang">
                            <th><span id="lang_097">System Language</span></th>
                            <td><span data-bind="SysLang"></span></td>
                        </tr>
                        <tr id="tr_CodePage">
                            <th><span id="lang_098">Code Page</span></th>
                            <td><span data-bind="CodePage"></span></td>
                        </tr>
                        <tr id="tr_Processes">
                            <th><span id="lang_110">Processes</span></th>
                            <td><span data-bind="Processes"></span></td>
                        </tr>
                    </tbody>
                </table>
            </div>
        </div>
    </div>

    <div id="block_hardware" class="col-lg-6" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_010">Hardware Information</span></div>
            <div class="panel-body">
                <table id="hardware" class="table table-hover table-condensed noborderattop">
                    <tbody id="hardware-data">
                    </tbody>
                </table>
            </div>
        </div>
    </div>

    <div id="block_memory" class="col-lg-12" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_027">Memory Usage</span></div>
            <div class="panel-body">
                <div class="table-responsive">
                    <table id="memory" class="table table-hover table-condensed">
                        <thead>
                            <tr>
                                <th><span id="lang_034-u01">Type</span></th>
                                <th><span id="lang_033-u02">Usage</span></th>
                                <th class="rightCell"><span id="lang_035-u03">Free</span></th>
                                <th class="rightCell"><span id="lang_036-u04">Used</span></th>
                                <th class="rightCell"><span id="lang_037-u05">Size</span></th>
                            </tr>
                        </thead>
                        <tbody id="memory-data">
                            <tr>
                                <th><span data-bind="Type"></span></th>
                                <td><span data-bind="Usage"></span></td>
                                <td class="rightCell"><span data-bind="Free"></span></td>
                                <td class="rightCell"><span data-bind="Used"></span></td>
                                <td class="rightCell"><span data-bind="Total"></span></td>
                            </tr>
                        </tbody>
                        <tfoot id="swap-data" style="display:none;">
                            <tr>
                                <th><span data-bind="Name"></span></th>
                                <td><span data-bind="Usage"></span></td>
                                <td class="rightCell"><span data-bind="Free"></span></td>
                                <td class="rightCell"><span data-bind="Used"></span></td>
                                <td class="rightCell"><span data-bind="Total"></span></td>
                            </tr>
                        </tfoot>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_filesystem" class="col-lg-12" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_030">Mounted Filesystems</span></div>
            <div class="panel-body">
                <div class="table-responsive">
                    <table id="filesystem" class="table table-hover table-condensed sortable">
                        <thead>
                            <tr>
                                <th id="filesystem_MountPoint"><span id="lang_031">Mountpoint</span></th>
                                <th><span id="lang_034-u06">Type</span></th>
                                <th><span id="lang_032">Partition</span></th>
                                <th><span id="lang_033-u07">Usage</span></th>
                                <th class="rightCell sorttable_numeric"><span id="lang_035-u08">Free</span></th>
                                <th class="rightCell sorttable_numeric"><span id="lang_036-u09">Used</span></th>
                                <th class="rightCell sorttable_numeric"><span id="lang_037-u10">Size</span></th>
                            </tr>
                        </thead>
                        <tbody id="filesystem-data">
                            <tr>
                                <th><span data-bind="MountPoint"></span></th>
                                <td><span data-bind="FSType"></span></td>
                                <td><span data-bind="Name"></span></td>
                                <td><span data-bind="Percent"></span></td>
                                <td class="rightCell"><span data-bind="Free"></span></td>
                                <td class="rightCell"><span data-bind="Used"></span></td>
                                <td class="rightCell"><span data-bind="Total"></span></td>
                            </tr>
                        </tbody>
                        <tfoot id="filesystem-foot">
                            <tr>
                                <td></td>
                                <td></td>
                                <th><span id="lang_038">Totals</span></th>
                                <th><span data-bind="Percent"></span></th>
                                <th class="rightCell"><span data-bind="Free"></span></th>
                                <th class="rightCell"><span data-bind="Used"></span></th>
                                <th class="rightCell"><span data-bind="Total"></span></th>
                            </tr>
                        </tfoot>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_network" class="col-lg-6" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_021">Network Interface</span></div>
            <div class="panel-body">
                <div class="table-responsive">
                    <table id="network" class="table table-hover table-condensed">
                        <thead>
                            <tr>
                                <th style="width:60%"><span id="lang_022">Device</span></th>
                                <th class="rightCell"><span id="lang_023">Receive</span></th>
                                <th class="rightCell"><span id="lang_024">Send</span></th>
                                <th class="rightCell"><span id="lang_025">Err/<wbr>Drop</span></th>
                            </tr>
                        </thead>
                        <tbody id="network-data"></tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_voltage" class="col-lg-6" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_052">Voltage</span></div>
            <div class="panel-body">
                <div class="table-responsive">
                    <table id="voltage" class="table table-hover table-condensed">
                        <thead>
                            <tr>
                                <th><span id="lang_059-u11">Label</span></th>
                                <th class="rightCell"><span id="lang_054-u12">Value</span></th>
                                <th class="rightCell"><span id="lang_055-u13">Min</span></th>
                                <th class="rightCell"><span id="lang_056-u14">Max</span></th>
                            </tr>
                        </thead>
                        <tbody id="voltage-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                                <td class="rightCell"><span data-bind="Min"></span></td>
                                <td class="rightCell"><span data-bind="Max"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_current" class="col-lg-6" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_105">Current</span></div>
            <div class="panel-body">
                <div class="table-responsive">
                    <table id="current" class="table table-hover table-condensed">
                        <thead>
                            <tr>
                                <th><span id="lang_059-u15">Label</span></th>
                                <th class="rightCell"><span id="lang_054-u16">Value</span></th>
                                <th class="rightCell"><span id="lang_055-u17">Min</span></th>
                                <th class="rightCell"><span id="lang_056-u18">Max</span></th>
                            </tr>
                        </thead>
                        <tbody id="current-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                                <td class="rightCell"><span data-bind="Min"></span></td>
                                <td class="rightCell"><span data-bind="Max"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_temperature" class="col-lg-3" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_051">Temperatures</span></div>
            <div class="panel-body">
                <div class="table-responsive">
                    <table id="temperature" class="table table-hover table-condensed">
                        <thead>
                            <tr>
                                <th><span id="lang_059-u19">Label</span></th>
                                <th class="rightCell"><span id="lang_054-u20">Value</span></th>
                                <th class="rightCell"><span id="lang_058-u21">Limit</span></th>
                            </tr>
                        </thead>
                        <tbody id="temperature-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                                <td class="rightCell"><span data-bind="Max"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_fans" class="col-lg-3" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_053">Fans</span></div>
            <div class="panel-body">
                <div class="table-responsive">
                    <table id="fans" class="table table-hover table-condensed">
                        <thead>
                            <tr>
                                <th><span id="lang_059-u22">Label</span></th>
                                <th class="rightCell"><span id="lang_054-u23">Value</span></th>
                                <th class="rightCell"><span id="lang_055-u24">Min</span></th>
                            </tr>
                        </thead>
                        <tbody id="fans-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                                <td class="rightCell"><span data-bind="Min"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_power" class="col-lg-3" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_102">Power</span></div>
            <div class="panel-body">
                <div class="table-responsive">
                    <table id="power" class="table table-hover table-condensed">
                        <thead>
                            <tr>
                                <th><span id="lang_059-u25">Label</span></th>
                                <th class="rightCell"><span id="lang_054-u26">Value</span></th>
                                <th class="rightCell"><span id="lang_058-u27">Limit</span></th>
                            </tr>
                        </thead>
                        <tbody id="power-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                                <td class="rightCell"><span data-bind="Max"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_other" class="col-lg-6" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_121">Other</span></div>
            <div class="panel-body">
                <div class="table-responsive">
                    <table id="other" class="table table-hover table-condensed">
                        <thead>
                            <tr>
                                <th><span id="lang_059-u28">Label</span></th>
                                <th class="rightCell"><span id="lang_054-u29">Value</span></th>
                            </tr>
                        </thead>
                        <tbody id="other-data">
                            <tr>
                                <th><span data-bind="Label"></span></th>
                                <td class="rightCell"><span data-bind="Value"></span></td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>

    <div id="block_ups" class="col-lg-6" style="display:none;">
        <div class="panel panel-primary">
            <div class="panel-heading"><span id="lang_068">UPS Information</span></div>
            <div class="panel-body">
                <table id="ups" class="table table-hover table-condensed noborderattop">
                    <tbody id="ups-data">
                    </tbody>
                </table>
            </div>
        </div>
    </div>

<?php foreach (CommonFunctions::getPlugins() as $plugin) :
if (file_exists(APP_ROOT . '/plugins/' . $plugin . '/' . $plugin . '_bootstrap.html')) : ?>
<?php include APP_ROOT. '/plugins/' . $plugin . '/' . $plugin . '_bootstrap.html'; ?>

<?php endif; endforeach; ?>

</div>

<!-- Errors details modal dialogs -->
<div class="modal fade" id="errors-dialog" tabindex="-1" role="dialog" aria-hidden="true">
    <div class="modal-dialog">
        <div class="modal-content">
            <div class="modal-header">
                <button type="button" class="close" data-dismiss="modal" aria-hidden="true">&times;</button>
                <h4 class="modal-title">Errors</h4>
            </div>
            <div class="modal-body">
                <ul id="errors" class="list-group">
                </ul>
            </div>
        </div>
        <!-- /.modal-content -->
    </div>
    <!-- /.modal-dialog -->
</div>
<!-- /.modal -->

</div>

<!--[if lt IE 9]>
    <script type="text/JavaScript" src="./js.php?name=jquery-1"></script>
<![endif]-->
<!--[if gte IE 9]><!-->
    <script type="text/JavaScript" src="./js.php?name=jquery"></script>
<!--<![endif]-->

<script type="text/JavaScript" src="./js.php?name=console-shim"></script>
<script type="text/JavaScript" src="./js.php?name=transparency"></script>
<script type="text/JavaScript" src="./js.php?name=sorttable"></script>
<script type="text/JavaScript" src="./js.php?name=jquery.treegrid"></script>
<script type="text/JavaScript" src="./js.php?name=bootstrap-modal"></script>

<!--[if lte IE 6]>
<script type="text/JavaScript" src="./js.php?name=jquery.ifixpng"></script>
<![endif]-->

<?php foreach (CommonFunctions::getPlugins() as $plugin) : ?>
<script type="text/JavaScript" src="./js.php?plugin=<?php echo $plugin ?>&amp;name=<?php echo $plugin ?>_bootstrap"></script>
<?php endforeach; ?>

<script type="text/JavaScript" src="./js.php?name=phpsysinfo_bootstrap"></script>

</body>
</html>
