body {
    font-size: 13px;
    padding-top: 80px;
    color: #c8c8c8;
    background-color: #272b30;
}

wbr {
    display: inline-block;
}

.table td.rightCell, .table th.rightCell {
    text-align: right;
}

.percent {
    text-align: center;
}

.panel-primary {
    background-color: #2e3338;
    border: none;
    box-shadow: 0 1px 2px rgba(0, 0, 0, 0.05);
    -moz-box-shadow: 0 1px 2px rgba(0, 0, 0, 0.05);
    -webkit-box-shadow: 0 1px 2px rgba(0, 0, 0, 0.05);
    -o-box-shadow: 0 1px 2px rgba(0, 0, 0, 0.05);
}

.ie8 .panel-primary {
}

.navbar {
    background-color: #3a3f44;
    border-color: #2b2e32;
}

.navbar-brand, .navbar-brand:hover {
    color: #fff;
}

.treegrid-span {
    display: table-cell;
    *float: left; /* ie6 ie7 */
}

.treegrid-spanbold {
    font-weight: bold;
    display: table-cell;
    *float: left; /* ie6 ie7 */
}

.treegrid-indent {
    width: 16px;
    height: 16px;
    display: table-cell;
    *float: left; /* ie6 ie7 */
}

.treegrid-expander {
    width: 0px;
    height: 16px;
    display: table-cell;
    *float: left; /* ie6 ie7 */
    cursor: pointer;
}

.normalicon {
    width: 16px;
    height: 16px;
    position: relative;
    top: 1px;
    display: table-cell;
    *float: left; /* ie6 ie7 */
    font-style: normal;
    font-weight: normal;
    line-height: 1;

    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
}
.normalicon-right:before {
    content: "\25ba";
}
.normalicon-down:before {
    content: "\25bc";
}
/* IE6 fix */
* html .normalicon-right {
    background: url('../gfx/right_gray.gif') no-repeat;
    height: 14px;
}
* html .normalicon-down {
    background: url('../gfx/down_gray.gif') no-repeat;
    height: 14px;
}
/* IE7 fix */
*+html .normalicon-right {
    background: url('../gfx/right_gray.gif') no-repeat;
    height: 14px;
}
*+html .normalicon-down {
    background: url('../gfx/down_gray.gif') no-repeat;
    height: 14px;
}

.table-responsive {
    border: 0;
}

table.sortable thead th {
    cursor: pointer;
    position: relative;
    top: 0;
    left: 0;
}

table.sortable thead th:hover {
    background: #3e444c;
}

.table-hover > tbody > tr:hover {
    background-color: #49515a;
}

.table-hover > tfoot > tr:hover {
    background-color: #49515a;
}

table.borderless td, table.borderless th, table.noborderattop tr:first-child td, table.noborderattop tr:first-child th {
    border: none!important;
}

.table {
    background-color: #2e3338!important;
    margin :0;
}

.logo {
    cursor: pointer;
}

.errorbutton {
    cursor: pointer;
}

.select {
    color: #c8c8c8;
}

.template {
    cursor: pointer;
    color: #333;
}

.language {
    cursor: pointer;
    color: #333;
}

.panel-heading {
  background-color: #3e444c!important;
  border-color: #3e444c!important;
}

.progress {
    margin-bottom: 0;
    background-color: #1c1e22;
}

.progress-bar-success {
    background-color: #62c462;
}

.progress-bar-info {
    background-color: #5bc0de;
}

.progress-bar-warning {
    background-color: #f89406;
}

.progress-bar-danger {
    background-color: #ee5f5b;
}

.modal-content {
    background-color: #2e3338;
}

.list-group-item {
    background-color: #2e3338;
}

.reload {
    background-image: url("../gfx/reload.gif");
    vertical-align: middle;
    float: right;
    cursor: pointer;
    border: 0px;
    width: 16px;
    height: 16px;    
}

a, a:visited {
    color: #fff;
}

a:hover {
    text-decoration: underline;
    color: #fff;
}

.psihref, .psihref:visited {
    color: #fff;
}

.psihref:hover {
    text-decoration: underline;
    color: #fff;
}
