/*
 $Id: jstyle_green.css 518 2011-10-28 08:09:07Z namiltd $
 */
a {
    color:#c03000;
    text-decoration:none;
}

a:hover {
    text-decoration:underline;
}

* {
    margin:0;
    padding:0;
}

wbr {
    display: inline-block;
}

html {
    background-color:#f3f1e9;
    color:#666;
    font-size:100%;
    height:100%;
}

body {
    font-family:Verdana, "Bitstream Vera Sans";
    font-size:.75em;
    margin:0 auto;
    min-height:100%;
    overflow:auto;
    padding:20px 20px 0;
    position:relative;
    width:940px;
    _width: 945px; /* ie6 */
}

p {
    line-height:1.6;
    padding:4px 10px 2px;
    text-align:left;
    vertical-align:top;
}

h1 {
    color:#693;
    font-family:Georgia, serif;
    font-size:150%;
    font-weight:400;
    line-height:1.5em;
    margin:0 10px;
    _margin: 0 15px 0 10px; /* ie6 */
    padding:10px;
    text-align:center;
}

h2 {
    border-bottom:2px solid #693;
    color:#693;
    font-family:Georgia, serif;
    font-size:130%;
    font-weight:700;
    line-height:1.5em;
    padding:3px 10px;
    text-transform:uppercase;
}

th,td,h3 {
    font-size:100%;
    padding:4px 10px 2px;
    text-align:left;
    vertical-align:top;
}

table {
    width:100%;
}

#select {
    text-align:right;
}

#select select {
    width:100px;
}

#vitals,#network, #memory, #filesystem, #hardware, #temperature, #voltage, #fans, #power, #current, #other, #ups {
    float:left;
    margin:10px 0 0 10px;
    _margin: 10px 5px 0 5px; /* ie6 */
    padding:1px;
    width:451px;
}

#pciTable, #ideTable, #scsiTable, #usbTable, #tbTable, #i2cTable {
    padding:0 30px;
}

#memory,#filesystem {
    width:915px;
}

#filesystemTable thead tr th {
    cursor:pointer;
}

#footer {
    clear:both;
    color:#5C5C5C;
    font-size:80%;
    line-height:18px;
    margin:12px;
    padding:13px 25px;
    text-align:center;
}

.plugin {
    float: left;
    margin: 10px 0 0 10px;
    _margin: 10px 5px 0 5px; /* ie6 */
    padding: 1px;
}

.even {
    background:#EBE8DA;
}

.bar {
    background-color:#C1DC70;
}

.barwarn {
    background-color:#DC8B70;
}

.right {
    padding-right:20px;
    text-align:right;
}

.treeimg {
    display: table-cell;
    *float: left; /* ie6 ie7 */
    vertical-align: top;
}

.treespan {
    display: table-cell;
    *float: left; /* ie6 ie7 */
    vertical-align: middle;
}

.treespanbold {
    font-weight: bold;
    display: table-cell;
    *float: left; /* ie6 ie7 */
    vertical-align: middle;
}

.tree tr td {
    cursor: auto;
}
